﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using HeuristicLab.Services.Hive.DataAccess;

namespace HeuristicLab.Services.Hive.Statistics.Controllers {
  public class AdminDataController : Controller {

    public JsonResult UserList()
    {
      using (var db = new HiveDataContext(Settings.Default.HeuristicLab_Hive_LinqConnectionString))
      {
        var allUsers = (from users in db.DimUsers
                            select new { users.Name })
                            .ToList();

        List<string> UserNames = new List<string>();
        allUsers.ForEach(u => UserNames.Add(u.Name));

        return Json(UserNames, JsonRequestBehavior.AllowGet);
      }
    }

    public JsonResult TaskStats(DateTime start, DateTime end, string userName=null) {
      using (var db = new HiveDataContext(Settings.Default.HeuristicLab_Hive_LinqConnectionString)) {
        var taskStats = (from tasks in db.FactTasks
                        join jobs in db.DimJobs
                          on tasks.JobId equals jobs.JobId
                        where tasks.StartTime >= start &&
                        tasks.EndTime < end &&
                        (string.IsNullOrEmpty(userName) || jobs.UserName == userName)
                        select new { tasks.TotalWaitingTime, tasks.TotalTransferTime, tasks.TotalRuntime, tasks.NumCalculationRuns, 
                          tasks.CoresRequired, tasks.MemoryRequired})
                          .ToList();

        List<KeyValuePair<string, double>> FullStats = new List<KeyValuePair<string, double>>();
        FullStats.Add(new KeyValuePair<string, double>("Total Waiting Time",taskStats.Sum(w => w.TotalWaitingTime)));
        FullStats.Add(new KeyValuePair<string, double>("Total Transfer Time", taskStats.Sum(t => t.TotalTransferTime)));
        FullStats.Add(new KeyValuePair<string, double>("Total Runtime", taskStats.Sum(r => r.TotalRuntime)));
        FullStats.Add(new KeyValuePair<string, double>("Number Calculation Runs", taskStats.Sum(n => n.NumCalculationRuns)));
        FullStats.Add(new KeyValuePair<string, double>("Cores Required", taskStats.Sum(c => c.CoresRequired)));
        FullStats.Add(new KeyValuePair<string, double>("Memory Required", taskStats.Sum(m => m.MemoryRequired)));

        return Json(FullStats, JsonRequestBehavior.AllowGet);
      }
    }

    public JsonResult MoreTaskInfo(string taskId) {
      using (var db = new HiveDataContext(Settings.Default.HeuristicLab_Hive_LinqConnectionString))
      {
        var moreInfo = (from tasks in db.FactTasks
                         where tasks.TaskId.ToString() == taskId
                         select new
                         {
                           tasks.TotalWaitingTime,
                           tasks.TotalTransferTime,
                           tasks.TotalRuntime,
                           tasks.NumCalculationRuns,
                           tasks.CoresRequired,
                           tasks.MemoryRequired,
                         })
                          .ToList();

        List<KeyValuePair<string, double>> AllInfo = new List<KeyValuePair<string, double>>();
        AllInfo.Add(new KeyValuePair<string, double>("Number Calculation Runs", moreInfo.Sum(n => n.NumCalculationRuns)));
        AllInfo.Add(new KeyValuePair<string, double>("Cores Required", moreInfo.Sum(c => c.CoresRequired)));
        AllInfo.Add(new KeyValuePair<string, double>("Memory Required", moreInfo.Sum(m => m.MemoryRequired)));

        return Json(AllInfo, JsonRequestBehavior.AllowGet);
      }
    }

    public JsonResult MoreSlaveInfo(string slaveId) {
      using (var db = new HiveDataContext(Settings.Default.HeuristicLab_Hive_LinqConnectionString))
      {
        var moreInfo = (from slaves in db.FactClientInfos
                        where slaves.ClientId.ToString() == slaveId
                        select new
                        {
                          slaves.Time,
                          slaves.TotalTimeIdle,
                          slaves.TotalTimeCalculating,
                          slaves.TotalTimeTransferring,
                          slaves.TotalTimeOffline,
                          slaves.TotalTimeUnavailable
                        }).ToList();

        List<List<KeyValuePair<string, string>>> AllInfo = new List<List<KeyValuePair<string, string>>>();
        List<List<KeyValuePair<string, string>>> DistinctInfo = new List<List<KeyValuePair<string, string>>>();
        for(var i=0; i < moreInfo.Count; i++) {
          List<KeyValuePair<string, string>> newRow = new List<KeyValuePair<string, string>>();
          newRow.Add(new KeyValuePair<string, string>("Time", moreInfo.ElementAt(i).Time.ToShortDateString() + " " + moreInfo.ElementAt(i).Time.ToShortTimeString()));
          newRow.Add(new KeyValuePair<string, string>("Time Idle", moreInfo.ElementAt(i).TotalTimeIdle.ToString()));
          newRow.Add(new KeyValuePair<string, string>("Time Calculating", moreInfo.ElementAt(i).TotalTimeCalculating.ToString()));
          newRow.Add(new KeyValuePair<string, string>("Time Transferring", moreInfo.ElementAt(i).TotalTimeTransferring.ToString()));
          newRow.Add(new KeyValuePair<string, string>("Time Offline", moreInfo.ElementAt(i).TotalTimeOffline.ToString()));
          newRow.Add(new KeyValuePair<string, string>("Time Unavailable", moreInfo.ElementAt(i).TotalTimeUnavailable.ToString()));
          AllInfo.Add(newRow);
        }

        //Ensures that only records with changes in Traffic or Time fields are included
        List<KeyValuePair<string, string>> lastRecord = null;
        //Initilized to true to ensure first record is automatically included
        bool isDistinct = true;
        foreach (List<KeyValuePair<string, string>> row in AllInfo) {
          //Ignores check for first row
          if (lastRecord != null) {
            isDistinct = false;
            //Cycles through values of current row and lastRecord
            for (var j=0; j < row.Count; j++) {
              //Disregards time field from distinct checks
              if (row.ElementAt(j).Key != "Time") {
                //If values differ then row is distinct, break loop
                if (row.ElementAt(j).Value != lastRecord.ElementAt(j).Value) {
                  isDistinct = true;
                  break;
                }
              }
            }
          }
          lastRecord = row;
          if (isDistinct) {
            DistinctInfo.Add(row);
          }
        }

        return Json(DistinctInfo, JsonRequestBehavior.AllowGet);
      }
    }
  }
}
