﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using HeuristicLab.Services.Hive.DataAccess;

namespace HeuristicLab.Services.Hive.Statistics.Controllers {
  public class ExceptionDataController : Controller {

    public JsonResult TaskExceptions(string userName, DateTime? start = null, DateTime? end = null, string jobId = null, string taskState = null, string slaveId = null)
    {
      using (var db = new HiveDataContext(Settings.Default.HeuristicLab_Hive_LinqConnectionString)) {
        TaskState? state = GetTaskState(taskState);
        var data =
        (from tasks in db.FactTasks
          join jobs in db.DimJobs
            on tasks.JobId equals jobs.JobId
          where tasks.Exception != null && 
          !tasks.Exception.Equals(string.Empty) &&
          (string.IsNullOrEmpty(userName) || jobs.UserName == userName) &&
          (!start.HasValue || tasks.StartTime >= start) &&
          (!end.HasValue || tasks.EndTime < end) &&
          (string.IsNullOrEmpty(jobId) || tasks.JobId.ToString() == jobId) &&
          (string.IsNullOrEmpty(taskState) || tasks.TaskState == state) &&
          (string.IsNullOrEmpty(slaveId) || tasks.LastClientId.ToString() == slaveId)
          select new
          {
            TaskID = tasks.TaskId,
            ErrorMessage = tasks.Exception,
            StartDate = tasks.StartTime
          }).OrderByDescending(s => s.StartDate).ToList();

        List<string> Task = new List<string>();
        List<string> Error = new List<string>();
        data.ForEach(t => Task.Add(t.TaskID.ToString()));
        data.ForEach(e => Error.Add(e.ErrorMessage));

        return Json(new KeyValuePair<List<string>, List<string>>(Task, Error), JsonRequestBehavior.AllowGet);
      }
    }

    public JsonResult SlaveExceptions(string limit, DateTime? start = null, DateTime? end = null, string clientId = null) {
      using (var db = new HiveDataContext(Settings.Default.HeuristicLab_Hive_LinqConnectionString)) {
        var data =
        (from tasks in db.FactTasks
         join jobs in db.DimJobs
           on tasks.JobId equals jobs.JobId
         join clients in db.DimClients
           on tasks.LastClientId equals clients.Id
         where tasks.Exception != null &&
          !tasks.Exception.Equals(string.Empty) && 
          (!start.HasValue || tasks.StartTime >= start) &&
         (!end.HasValue || tasks.EndTime < end) &&
         (string.IsNullOrEmpty(clientId) || tasks.LastClientId.ToString() == clientId)
         select new {
           TaskID = tasks.TaskId,
           ErrorMessage = tasks.Exception,
           ClientID = tasks.LastClientId,
           ClientName = clients.Name,
           UserName = jobs.UserName,
           JobName = jobs.JobName,
           StartDate = tasks.StartTime
         }).OrderByDescending(s => s.StartDate).ToList();

        List<List<string>> results = new List<List<string>>();

        for (int i = 0; i < data.Count; i++) {
          results.Add(
            new List<string> { data[i].TaskID.ToString(), data[i].ErrorMessage, data[i].ClientID.ToString(), data[i].ClientName,
              data[i].UserName, data[i].JobName, data[i].StartDate.ToString("dd/MM/yyyy HH:mm")
            }
          );
        }
        return Json(results, JsonRequestBehavior.AllowGet);
      }
    }

    private static TaskState? GetTaskState(string taskState) {
      TaskState? state;
      switch (taskState) {
        case "Aborted":
          state = TaskState.Aborted;
          break;
        case "Calculating":
          state = TaskState.Calculating;
          break;
        case "Failed":
          state = TaskState.Failed;
          break;
        case "Finished":
          state = TaskState.Finished;
          break;
        case "Offline":
          state = TaskState.Offline;
          break;
        case "Paused":
          state = TaskState.Paused;
          break;
        case "Transferring":
          state = TaskState.Transferring;
          break;
        case "Waiting":
          state = TaskState.Waiting;
          break;
        default :
          state = null;
          break;
      }
      return state;
    }
  }
}
